#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history https://hg.reportlab.com/hg-public/reportlab/log/tip/src/reportlab/platypus/__init__.py
__version__='3.5.20'
__doc__='''Page Layout and Typography Using Scripts" - higher-level framework for flowing documents'''

from .flowables import *
from .frames import *
from .multicol import *
from .paragraph import *
from .paraparser import *
from .tables import *
from .doctemplate import *
from .xpreformatted import *
